#!/usr/bin/python3
# -*- coding: utf-8 -*-

   #################################
   #                               #
   #   Equation de la chaleur 1D   #
   #                               #
   #################################

import time
import numpy as np
import matplotlib.pyplot as plt

## INITIALISATION ##

# dimensions
L = 1.  # longueur du cylindre
D = 1.   # duree

# nombre de points de discretisation (espace et temps)
NbL = 51
NbD = 10001

# coefficients de diffusion
alpha = 1
alpha2 = 0.1

# pas de discretisation
dx, dt = L / (NbL-1), D / (NbD-1)

# variables
i = 0
m = int(NbL/2)  # indice milieu
Beta = alpha*dt / ( dx**2 )
Beta2 = alpha2*dt / ( dx**2 )   # selon le coeff de diffusion de la 2e barre

print("Beta={} ; Beta2={}".format(Beta, Beta2))

# grille espace
X = np.linspace(0.,L,NbL)  

# conditions de Dirichlet
T = 50.*np.ones(NbL)   # temperature par défaut de la barre au temps i*dt
T[0:m]=25.
T[0] = 0.
T[-1] = 0.

Temp = np.zeros(NbL)

## PROGRAMME PRINCIPAL ##

plt.figure()
plt.plot(X, T, linewidth=1, label="iter%1.0f" % i)

t = time.time()
if False :  # code interprete
   while i < NbD :
      for j in range(1,m):  # condition aux bords (Dirichlet)
         Temp[j] = Beta * ( T[j-1] - 2 * T[j] + T[j+1] )
      for j in range(m,NbL-1):  # condition aux bords (Dirichlet)
         Temp[j] = Beta2 * ( T[j-1] - 2 * T[j] + T[j+1] )
      for j in range(1,NbL-1):
         T[j] += Temp[j]
      i += 1
   #   if np.log10(i) in range(6):   # i % 2000==0:
      if i in [1,5,10,50,100,500,1000,5000,10000]:
         plt.plot(X, T, linewidth=1, label="iter%1.0f" % i)

if True :  # code compile
   while i < NbD :
   #   Temp[1:-1] = Beta*( T[:-2] - 2*T[1:-1] + T[2:] )
      Temp[1:m] = Beta*( T[:m-1] - 2*T[1:m] + T[2:m+1] )
      Temp[m:-1] = Beta2*( T[m-1:-2] - 2*T[m:-1] + T[m+1:] )
      T[1:-1] += Temp[1:-1]
   
      i += 1
   #   if np.log10(i) in range(6):   # i % 2000==0:
      if i in [1,5,10,50,100,500,1000,5000,10000]:
         plt.plot(X, T, linewidth=1, label="iter%1.0f" % i)

t = time.time()-t
print("Nb iterations : %1.0f," %i, "temps : %1.4f" %t)

plt.legend()
plt.show()

# fin du fichier EquationChaleur1D.py
