#!/usr/bin/python3
# -*- coding: utf-8 -*-

   #################################
   #                               #
   #   Equation de la chaleur 2D   #
   #                               #
   #################################

import time
import numpy as np
import matplotlib.pyplot as plt

## INITIALISATION ##

# dimensions du domaine
Lx, Ly = 1., 1.

# nombre de points
NbX, NbY = 51, 51

# pas de discretisation
dx, dy = Lx / (NbX-1), Ly / (NbY-1)

# grille du domaine
X = np.linspace(0,Lx,NbX)
Y = np.linspace(0,Ly,NbY)

# conditions aux bords
Tg, Td = 0., 0.
Tb, Th = 100., 100.

# constantes du schéma (1/4 si dx=dy)
a = dx**2 / ( 2*(dx**2+dy**2) )
b = dy**2 / ( 2*(dx**2+dy**2) )

# constantes d'arret
MaxErreur = .001
MaxIteration = 5*10**4

## PROGRAMME PRINCIPAL ##
if False :  # code interprete
   t = time.time()
   
   # initialisation des matrices Temperature (k et k-1)
   T = np.zeros((NbX,NbY))
   for i in range(NbX):
      T[i,0] = Tb
      T[i,-1] = Th
   for j in range(NbY):
      T[0,j] = Tg
      T[-1,j] = Td
   TT = np.copy(T)
   k = 0
   erreur = 1.
   while erreur > MaxErreur and k < MaxIteration :
      erreur = 0
      for i in range(1,NbX-1):
         for j in range(1,NbY-1):
            T[i,j] = b * ( TT[i+1,j]+T[i-1,j] ) + a* ( TT[i,j+1]+T[i,j-1] )
            erreur = max( abs( T[i,j]-TT[i,j] ), erreur)
      TT = np.copy(T)
      k += 1
   #   if k%100==0: print("iteration no %1.0f," %k, "erreur %1.3f" %erreur)
   t = time.time()-t
   print("Nb iterations : %1.0f," %k, "Erreur : %1.4f," %erreur, "temps : %1.4f" %t)
   
   plt.figure()
   plt.pcolormesh(X, Y, np.transpose(T), shading='flat')
   plt.colorbar()
   plt.axis('image')
   plt.show()

if True :   # code compile
   t = time.time()

   T = np.zeros((NbX,NbY))
   T[:,0] = Tb
   T[:,-1] = Th
   T[0,:] = Tg
   T[-1,:] = Td
   TT = np.copy(T)
   
   k = 0
   erreur = 1.
   while erreur > MaxErreur and k < MaxIteration :
      erreur = 0
      T[1:-1,1:-1] =  b * ( TT[:-2,1:-1]+T[2:,1:-1] ) + a* ( TT[1:-1,2:]+T[1:-1,:-2] )
      erreur = np.max(np.abs(T-TT))
      TT = np.copy(T)
      k += 1
   #   if k%100==0: print("iteration no %1.0f," %k, "erreur %1.3f" %erreur)
   t = time.time()-t
   print("Nb iterations : %1.0f," %k, "Erreur : %1.4f," %erreur, "temps : %1.4f" %t)


plt.figure()
plt.pcolormesh(X, Y, np.transpose(T), shading='flat')   # T[j,i]
plt.colorbar()
plt.axis('image')
plt.show()

# fin du fichier EquationChaleur2D.py
